@@ -11,8 +11,11 @@ import sys
 import warnings
 from pathlib import Path
 from textwrap import indent
+from unittest import SkipTest
 
 import pytest
+from _pytest.outcomes import OutcomeException  # Private API, but been around since 3.7
+from _pytest.doctest import _get_continue_on_failure  # Since 3.5, still in 7.3
 from packaging.version import Version
 
 from pytest_doctestplus.utils import ModuleChecker
@@ -85,6 +88,10 @@ def pytest_addoption(parser):
                           "features not found in the normal doctest "
                           "plugin")
 
+    parser.addoption("--doctest-ufunc", action="store_true",
+                     help="enable running doctests in docstrings of Numpy "
+                          "ufuncs")
+
     parser.addoption("--doctest-rst", action="store_true",
                      help=(
                          "Enable running doctests in .rst documentation. "
@@ -121,6 +128,9 @@ def pytest_addoption(parser):
     parser.addini("doctest_plus", "enable running doctests with additional "
                                   "features not found in the normal doctest plugin")
 
+    parser.addini("doctest_ufunc", "enable running doctests in docstrings of "
+                                   "Numpy ufuncs")
+
     parser.addini("doctest_norecursedirs",
                   "like the norecursedirs option but applies only to doctest "
                   "collection", type="args", default=())
@@ -157,11 +167,27 @@ def get_optionflags(parent):
     return flag_int
 
 
+def _is_numpy_ufunc(method):
+    try:
+        import numpy as np
+    except ModuleNotFoundError:
+        # If Numpy is not installed, then there can't be any ufuncs!
+        return False
+    while True:
+        try:
+            method = method.__wrapped__
+        except AttributeError:
+            break
+    return isinstance(method, np.ufunc)
+
+
 def pytest_configure(config):
     doctest_plugin = config.pluginmanager.getplugin('doctest')
     run_regular_doctest = config.option.doctestmodules and not config.option.doctest_plus
     use_doctest_plus = config.getini(
         'doctest_plus') or config.option.doctest_plus or config.option.doctest_only
+    use_doctest_ufunc = config.getini(
+        'doctest_ufunc') or config.option.doctest_ufunc
     if doctest_plugin is None or run_regular_doctest or not use_doctest_plus:
         return
 
@@ -234,9 +260,14 @@ def pytest_configure(config):
             options = get_optionflags(self) | FIX
 
             # uses internal doctest module parsing mechanism
-            finder = DocTestFinderPlus()
-            runner = doctest.DebugRunner(
-                verbose=False, optionflags=options, checker=OutputChecker())
+            finder = DocTestFinderPlus(doctest_ufunc=use_doctest_ufunc)
+            runner = DebugRunnerPlus(
+                verbose=False,
+                optionflags=options,
+                checker=OutputChecker(),
+                # Helper disables continue-on-failure when debugging is enabled
+                continue_on_failure=_get_continue_on_failure(config),
+            )
 
             for test in finder.find(module):
                 if test.examples:  # skip empty doctests
@@ -297,8 +328,10 @@ def pytest_configure(config):
 
             optionflags = get_optionflags(self) | FIX
 
-            runner = doctest.DebugRunner(
-                verbose=False, optionflags=optionflags, checker=OutputChecker())
+            runner = DebugRunnerPlus(
+                verbose=False, optionflags=optionflags, checker=OutputChecker(),
+                continue_on_failure=_get_continue_on_failure(self.config),
+            )
 
             parser = DocTestParserPlus()
             test = parser.get_doctest(text, globs, filepath, filename, 0)
@@ -476,16 +509,12 @@ class DoctestPlus(object):
         """
         if PYTEST_GE_7_0:
             dirpath = Path(path).parent
-        else:
-            dirpath = path.dirpath()
-
-        try:
-            collect_ignore = config._getconftest_pathlist("collect_ignore", path=dirpath)
-        except TypeError:
-            # Pytest 7.0+
             collect_ignore = config._getconftest_pathlist("collect_ignore",
                                                           path=dirpath,
                                                           rootpath=config.rootpath)
+        else:
+            dirpath = path.dirpath()
+            collect_ignore = config._getconftest_pathlist("collect_ignore", path=dirpath)
 
         # The collect_ignore conftest.py variable should cause all test
         # runners to ignore this file and all subfiles and subdirectories
@@ -525,7 +554,7 @@ class DoctestPlus(object):
                     self._ignore_paths.append(path)
                     break
 
-        return False
+        # None = Let other plugins decide the outcome.
 
     def pytest_collect_file(self, path, parent):
         """Implements an enhanced version of the doctest module from py.test
@@ -616,6 +645,10 @@ class DocTestFinderPlus(doctest.DocTestFinder):
     _import_cache = {}
     _module_checker = ModuleChecker()
 
+    def __init__(self, *args, doctest_ufunc=False, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._doctest_ufunc = doctest_ufunc
+
     @classmethod
     def check_required_modules(cls, mods):
         """Check that modules in `mods` list are available.
@@ -644,13 +677,23 @@ class DocTestFinderPlus(doctest.DocTestFinder):
 
     def find(self, obj, name=None, module=None, globs=None, extraglobs=None):
         tests = doctest.DocTestFinder.find(self, obj, name, module, globs, extraglobs)
+
+        if name is None and hasattr(obj, '__name__'):
+            name = obj.__name__
+        else:
+            raise ValueError(
+                "DocTestFinder.find: name must be given when obj.__name__ doesn't exist: "
+                f"{type(obj)!r}"
+            )
+
+        if self._doctest_ufunc:
+            for ufunc_name, ufunc_method in obj.__dict__.items():
+                if _is_numpy_ufunc(ufunc_method):
+                    tests += doctest.DocTestFinder.find(
+                        self, ufunc_method, f'{name}.{ufunc_name}',
+                        module=obj, globs=globs, extraglobs=extraglobs)
+
         if hasattr(obj, '__doctest_skip__') or hasattr(obj, '__doctest_requires__'):
-            if name is None and hasattr(obj, '__name__'):
-                name = obj.__name__
-            else:
-                raise ValueError("DocTestFinder.find: name must be given "
-                                 "when obj.__name__ doesn't exist: {!r}"
-                                 .format((type(obj),)))
 
             def test_filter(test):
                 for pat in getattr(obj, '__doctest_skip__', []):
@@ -675,3 +718,26 @@ class DocTestFinderPlus(doctest.DocTestFinder):
             tests = list(filter(test_filter, tests))
 
         return tests
+
+
+class DebugRunnerPlus(doctest.DebugRunner):
+    def __init__(self, checker=None, verbose=None, optionflags=0, continue_on_failure=True):
+        super().__init__(checker=checker, verbose=verbose, optionflags=optionflags)
+        self.continue_on_failure = continue_on_failure
+
+    def report_failure(self, out, test, example, got):
+        failure = doctest.DocTestFailure(test, example, got)
+        if self.continue_on_failure:
+            out.append(failure)
+        else:
+            raise failure
+
+    def report_unexpected_exception(self, out, test, example, exc_info):
+        cls, exception, traceback = exc_info
+        if isinstance(exception, (OutcomeException, SkipTest)):
+            raise exception
+        failure = doctest.UnexpectedException(test, example, exc_info)
+        if self.continue_on_failure:
+            out.append(failure)
+        else:
+            raise failure
